@@ -1,17 +1,20 @@
+/* David Leonard <david.leonard@csee.uq.edu.au>, 1999. Public domain. */
 
 /* 
  * LDAPObject - wrapper around an LDAP* context
- * $Id: LDAPObject.c,v 1.9 1997/12/01 22:46:09 leonard Exp $
+ * $Id: LDAPObject.c,v 1.14 1999/11/26 04:43:16 leonard Exp $
  */
 
 #include <math.h>
-#include <sys/time.h>
+#include <limits.h>
 #include "common.h"
 #include "errors.h"
 #include "constants.h"
 #include "LDAPObject.h"
 #include "message.h"
 
+#include "Python.h"
+
 /* constructor */
 
 LDAPObject*
@@ -257,6 +260,11 @@ attrs_from_List( PyObject *attrlist, char***attrsp ) {
 
     if (PyNone_Check( attrlist )) {
     	attrs = NULL;
+    } else if (PyString_Check( attrlist )) {
+	/* caught by John Benninghoff <johnb@netscape.com> */
+	PyErr_SetObject( PyExc_TypeError, Py_BuildValue("sO",
+		  "expected *list* of strings, not a string", attrlist ));
+        return 0;
     } else if (PySequence_Check( attrlist )) {
 	int len = PySequence_Length( attrlist );
 	int i;
@@ -323,6 +331,18 @@ l_ldap_unbind( LDAPObject* self, PyObject* args )
     return Py_None;
 }
 
+static char doc_unbind[] =
+"unbind_s() -> None\n"
+"unbind() -> int\n\n"
+"\tThis call is used to unbind from the directory, terminate\n"
+"\tthe current association, and free resources. Once called, the\n"
+"\tconnection to the LDAP server is closed and the LDAP object\n"
+"\tis invalid. Further invocation of methods on the object will\n"
+"\tyield an exception.\n"
+"\n"
+"\tThe unbind and unbind_s methods are identical, and are\n"
+"\tsynchronous in nature";
+
 /* ldap_abandon */
 
 static PyObject*
@@ -338,6 +358,14 @@ l_ldap_abandon( LDAPObject* self, PyObject* args )
     return Py_None;
 }
 
+static char doc_abandon[] =
+"abandon(msgid) -> None\n\n"
+"\tAbandons or cancels an LDAP operation in progress. The msgid should\n"
+"\tbe the message id of an outstanding LDAP operation as returned\n"
+"\tby the asynchronous methods search(), modify() etc.  The caller\n"
+"\tcan expect that the result of an abandoned operation will not be\n"
+"\treturned from a future call to result().";
+
 /* ldap_add */
 
 static PyObject *
@@ -363,6 +391,11 @@ l_ldap_add( LDAPObject* self, PyObject *args )
     return PyInt_FromLong(msgid);
 }
 
+static char doc_add[] =
+"add(dn, modlist) -> int\n\n"
+"\tThis function is similar to modify(), except that no operation\n"
+"\tinteger need be included in the tuples.";
+
 /* ldap_add_s */
 
 static PyObject *
@@ -392,6 +425,32 @@ l_ldap_add_s( LDAPObject* self, PyObject *args )
     return Py_None;
 }
 
+static char doc_bind[] =
+"bind(who, cred, method) -> int\n"
+"bind_s(who, cred, method) -> None\n"
+"simple_bind(who, passwd) -> int\n"
+"simple_bind_s(who, passwd) -> None\n"
+#ifdef WITH_KERBEROS
+"kerberos_bind_s(who) -> None\n"
+"kerberos_bind1(who) -> None\n"
+"kerberos_bind1_s(who) -> None\n"
+"kerberos_bind2(who) -> None\n"
+"kerberos_bind2_s(who) -> None\n\n"
+#endif
+"\tAfter an LDAP object is created, and before any other operations\n"
+"\tcan be attempted over the connection, a bind operation must\n"
+"\tbe performed.\n"
+"\n"
+"\tThis method attempts to bind with the LDAP server using either\n"
+"\tsimple authentication, or kerberos. The general method bind()\n"
+"\ttakes a third parameter, method which can be one of AUTH_SIMPLE,\n"
+"\tAUTH_KRBV41 or AUTH_KRBV42.  The cred parameter is ignored\n"
+"\tfor Kerberos authentication.\n"
+"\n"
+"\tKerberos authentication is only available if the LDAP library\n"
+"\tand the ldap module were both configured with --with-kerberos or\n"
+"\tcompiled with -DWITH_KERBEROS.";
+
 /* ldap_bind */
 
 static PyObject*
@@ -528,6 +587,21 @@ l_ldap_set_rebind_proc( LDAPObject* self, PyObject* args )
     return NULL;
 }
 
+static char doc_set_rebind_proc[] =
+"set_rebind_proc(func) -> None\n\n"
+"\tIf a referral is returned from the server, automatic re-binding\n"
+"\tcan be achieved by providing a function that accepts as an\n"
+"\targument the newly opened LDAP object and returns the tuple\n"
+"\t(who, cred, method).\n"
+"\n"
+"\tPassing a value of None for func will disable this facility.\n"
+"\n"
+"\tBecause of restrictions in the implementation, only one\n"
+"\trebinding function is supported at any one time. This method\n"
+"\tis only available if the module and library were compiled with\n"
+"\tsupport for it.";
+
+
 #endif /* LDAP_REFERRALS */
 
 /* ldap_simple_bind */
@@ -674,6 +748,19 @@ l_ldap_enable_cache( LDAPObject* self, PyObject* args )
     return Py_None;
 }
 
+static char doc_enable_cache[] =
+"enable_cache([timeout=NO_LIMIT, [maxmem=NO_LIMIT]]) -> None\n\n"
+"\tUsing a cache often greatly improves performance. By default\n"
+"\tthe cache is disabled. Specifying timeout in seconds is used\n"
+"\tto decide how long to keep cached requests. The maxmem value\n"
+"\tis in bytes, and is used to set an upper bound on how much\n"
+"\tmemory the cache will use. A value of NO_LIMIT for either\n"
+"\tindicates unlimited.  Subsequent calls to enable_cache()\n"
+"\tcan be used to adjust these parameters.\n"
+"\n"
+"\tThis and other caching methods are not available if the library\n"
+"\tand the ldap module were compiled with support for it.";
+
 /* ldap_disable_cache */
 
 static PyObject *
@@ -686,6 +773,12 @@ l_ldap_disable_cache( LDAPObject* self, PyObject *args )
     return Py_None;
 }
 
+static char doc_disable_cache[] =
+"disable_cache() -> None\n\n"
+"\tTemporarily disables use of the cache. New requests are\n"
+"\tnot cached, and the cache is not checked when returning\n"
+"\tresults. Cache contents are not deleted.";
+
 /* ldap_set_cache_options */
 
 static PyObject *
@@ -700,6 +793,15 @@ l_ldap_set_cache_options( LDAPObject* self, PyObject *args )
     return Py_None;
 }
 
+static char doc_set_cache_options[] =
+"set_cache_options(option) -> None\n\n"
+"\tChanges the caching behaviour. Currently supported options are\n"
+"\t    CACHE_OPT_CACHENOERRS, which suppresses caching of requests\n"
+"\t        that resulted in an error, and\n"
+"\t    CACHE_OPT_CACHEALLERRS, which enables caching of all requests.\n"
+"\tThe default behaviour is not to cache requests that result in\n"
+"\terrors, except those that result in a SIZELIMIT_EXCEEDED exception.";
+
 /* ldap_destroy_cache */
 
 static PyObject *
@@ -712,6 +814,10 @@ l_ldap_destroy_cache( LDAPObject* self, PyObject *args )
     return Py_None;
 }
 
+static char doc_destroy_cache[] =
+"destroy_cache() -> None\n\n"
+"\tTurns off caching and removed it from memory.";
+
 /* ldap_flush_cache */
 
 static PyObject *
@@ -724,6 +830,10 @@ l_ldap_flush_cache( LDAPObject* self, PyObject *args )
     return Py_None;
 }
 
+static char doc_flush_cache[] =
+"flush_cache() -> None\n\n"
+"\tDeletes the cache's contents, but does not affect it in any other way.";
+
 /* ldap_uncache_entry */
 
 static PyObject *
@@ -738,6 +848,11 @@ l_ldap_uncache_entry( LDAPObject* self, PyObject *args )
     return Py_None;
 }
 
+static char doc_uncache_entry[] =
+"uncache_entry(dn) -> None\n\n"
+"\tRemoves all cached entries that make reference to dn. This should be\n"
+"\tused, for example, after doing a modify() involving dn.";
+
 /* ldap_uncache_request */
 
 static PyObject *
@@ -752,6 +867,10 @@ l_ldap_uncache_request( LDAPObject* self, PyObject *args )
     return Py_None;
 }
 
+static char doc_uncache_request[] =
+"uncache_request(msgid) -> None\n\n"
+"\tRemove the request indicated by msgid from the cache.";
+
 #endif /* !NO_CACHE */
 
 /* ldap_compare */
@@ -789,6 +908,22 @@ l_ldap_compare_s( LDAPObject* self, PyObject *args )
     return PyInt_FromLong( result == LDAP_COMPARE_TRUE );
 }
 
+static char doc_compare[] =
+"compare(dn, attr, value) -> int\n"
+"compare_s(dn, attr, value) -> int\n\n"
+"\tPerform an LDAP comparison between the attribute named attr of\n"
+"\tentry dn, and the value value. The synchronous form returns 0\n"
+"\tfor false, or 1 for true.  The asynchronous form returns the\n"
+"\tmessage id of the initiates request, and the result of the\n"
+"\tasynchronous compare can be obtained using result().\n"
+"\n"
+"\tNote that this latter technique yields the answer by raising\n"
+"\tthe exception objects COMPARE_TRUE or COMPARE_FALSE.\n"
+"\n"
+"\tA design bug in the library prevents value from containing\n"
+"\tnul characters.";
+
+
 /* ldap_delete */
 
 static PyObject *
@@ -824,6 +959,13 @@ l_ldap_delete_s( LDAPObject* self, PyObject *args )
     return Py_None;
 }
 
+static char doc_delete[] =
+"delete(dn) -> int\n"
+"delete_s(dn) -> None\n\n"
+"\tPerforms an LDAP delete operation on dn. The asynchronous\n"
+"\tform returns the message id of the initiated request, and the\n"
+"\tresult can be obtained from a subsequent call to result().";
+
 /* ldap_modify */
 
 static PyObject *
@@ -878,6 +1020,24 @@ l_ldap_modify_s( LDAPObject* self, PyObject *args )
     return Py_None;
 }
 
+static char doc_modify[] =
+"modify(dn, modlist) -> int\n"
+"modify_s(dn, modlist) -> None\n\n"
+"\tPerforms an LDAP modify operation on an entry's attributes.\n"
+"\tdn is the DN of the entry to modify, and modlist is the list\n"
+"\tof modifications to make to the entry.\n"
+"\n"
+"\tEach element of the list modlist should be a tuple of the form\n"
+"\t(mod_op,mod_type,mod_vals), where mod_op is the operation (one\n"
+"\tof MOD_ADD, MOD_DELETE, or MOD_REPLACE), mod_type is a string\n"
+"\tindicating the attribute type name, and mod_vals is either\n"
+"\ta string value or a list of string values to add, delete or\n"
+"\treplace respectively.  For the delete operation, mod_vals may\n"
+"\tbe None indicating that all attributes are to be deleted.\n"
+"\n"
+"\tThe asynchronous modify() returns the message id of the\n"
+"\tinitiated request.";
+
 /* ldap_modrdn */
 
 static PyObject *
@@ -920,6 +1080,20 @@ l_ldap_modrdn_s( LDAPObject* self, PyObject *args )
     return Py_None;
 }
 
+static char doc_modrdn[] =
+"modrdn(dn, newrdn [,delold=1]) -> int\n"
+"modrdn_s(dn, newrdn [,delold=1]) -> None\n\n"
+"\tPerform a modify RDN operation. These routines take dn, the\n"
+"\tDN of the entry whose RDN is to be changed, and newrdn, the\n"
+"\tnew RDN to give to the entry. The optional parameter delold\n"
+"\tis used to specify whether the old RDN should be kept as\n"
+"\tan attribute of the entry or not.  The asynchronous version\n"
+"\treturns the initiated message id.\n"
+"\n"
+"\tThis actually corresponds to the modrdn2* routines in the\n"
+"\tC library.";
+
+
 /* ldap_result */
 
 static PyObject *
@@ -966,6 +1140,51 @@ l_ldap_result( LDAPObject* self, PyObject *args )
     }
 }
 
+static char doc_result[] =
+"result([msgid=RES_ANY [,all=1 [,timeout=-1]]]) -> (result_type, result_data)\n"
+"\n"
+"\tThis method is used to wait for and return the result of an\n"
+"\toperation previously initiated by one of the LDAP asynchronous\n"
+"\toperation routines (eg search(), modify(), etc.) They all\n"
+"\treturned an invocation identifier (a message id) upon successful\n"
+"\tinitiation of their operation. This id is guaranteed to be\n"
+"\tunique across an LDAP session, and can be used to request the\n"
+"\tresult of a specific operation via the msgid parameter of the\n"
+"\tresult() method.\n"
+"\n"
+"\tIf the result of a specific operation is required, msgid should\n"
+"\tbe set to the invocation message id returned when the operation\n"
+"\twas initiated; otherwise RES_ANY should be supplied.\n"
+"\n"
+"\tThe all parameter only has meaning for search() responses\n"
+"\tand is used to select whether a single entry of the search\n"
+"\tresponse should be returned, or to wait for all the results\n"
+"\tof the search before returning.\n"
+"\n"
+"\tA search response is made up of zero or more search entries\n"
+"\tfollowed by a search result. If all is 0, search entries will\n"
+"\tbe returned one at a time as they come in, via separate calls\n"
+"\tto result(). If all is 1, the search response will be returned\n"
+"\tin its entirety, i.e. after all entries and the final search\n"
+"\tresult have been received.\n"
+"\n"
+"\tThe method returns a tuple of the form (result_type,\n"
+"\tresult_data).  The result_type is a string, being one of:\n"
+"\t'RES_BIND', 'RES_SEARCH_ENTRY', 'RES_SEARCH_RESULT',\n"
+"\t'RES_MODIFY', 'RES_ADD', 'RES_DELETE', 'RES_MODRDN', or\n"
+"\t'RES_COMPARE'.\n"
+"\n"
+"\tThe constants RES_* are set to these strings, for convenience.\n"
+"\n"
+"\tSee search() for a description of the search result's\n"
+"\tresult_data, otherwise the result_data is normally meaningless.\n"
+"\n"
+"\tThe result() method will block for timeout seconds, or\n"
+"\tindefinitely if timeout is negative.  A timeout of 0 will effect\n"
+"\ta poll.  The timeout can be expressed as a floating-point value.\n"
+"\n"
+"\tIf a timeout occurs, the tuple (None, None) is returned.";
+
 /* ldap_search */
 
 static PyObject*
@@ -1046,6 +1265,62 @@ l_ldap_search_st( LDAPObject* self, PyObject* args )
     }
 }	
 
+static char doc_search[] =
+"search(base, scope, filter [,attrlist=None [,attrsonly=0]]) -> int\n"
+"search_s(base, scope, filter [,attrlist=None [,attrsonly=0]])\n"
+"search_st(base, scope, filter [,attrlist=None [,attrsonly=0 [,timeout=-1]]])\n"
+"\n"
+"\tPerform an LDAP search operation, with base as the DN of\n"
+"\tthe entry at which to start the search, scope being one of\n"
+"\tSCOPE_BASE (to search the object itself), SCOPE_ONELEVEL\n"
+"\t(to search the object's immediate children), or SCOPE_SUBTREE\n"
+"\t(to search the object and all its descendants).\n"
+"\n"
+"\tfilter is a string representation of the filter to\n"
+"\tapply in the search. Simple filters can be specified as\n"
+"\t'attribute_type=attribute_value'.  More complex filters are\n"
+"\tspecified using a prefix notation according to the following\n"
+"\tBNF:\n"
+"\n"
+"\t    filter     ::=  '(' filtercomp ')'\n"
+"\t    filtercomp ::=  and | or | not | simple\n"
+"\t    and        ::=  '&' filterlist\n"
+"\t    or         ::=  '|' filterlist\n"
+"\t    not        ::=  '!' filter\n"
+"\t    filterlist ::=  filter | filter filterlist\n"
+"\t    simple     ::=  attributetype filtertype attributevalue\n"
+"\t    filtertype ::=  '=' | '~=' | '<=' | '>='\n"
+"\n"
+"\tWhen using the asynchronous form and result(), the all parameter\n"
+"\taffects how results come in.  For all set to 0, result tuples\n"
+"\ttrickle in (with the same message id), and with the result type\n"
+"\tRES_SEARCH_ENTRY, until the final result which has a result\n"
+"\ttype of RES_SEARCH_RESULT and a (usually) empty data field.\n"
+"\tWhen all is set to 1, only one result is returned, with a\n"
+"\tresult type of RES_SEARCH_RESULT, and all the result tuples\n"
+"\tlisted in the data field.\n"
+"\n"
+"\tEach result tuple is of the form (dn,attrs), where dn is a\n"
+"\tstring containing the DN (distinguished name) of the entry, and\n"
+"\tattrs is a dictionary containing the attributes associated with\n"
+"\tthe entry.  The keys of attrs are strings, and the associated\n"
+"\tvalues are lists of strings.\n"
+"\n"
+"\tThe DN in dn is extracted using the underlying ldap_get_dn(),\n"
+"\twhich may raise an exception of the DN is malformed.\n"
+"\n"
+"\tIf attrsonly is non-zero, the values of attrs will be\n"
+"\tmeaningless (they are not transmitted in the result).\n"
+"\n"
+"\tThe retrieved attributes can be limited with the attrlist\n"
+"\tparameter.  If attrlist is None, all the attributes of each\n"
+"\tentry are returned.\n"
+"\n"
+"\tThe synchronous form with timeout, search_st(), will block\n"
+"\tfor at most timeout seconds (or indefinitely if timeout is\n"
+"\tnegative). A TIMEOUT exception is raised if no result is\n"
+"\treceived within the time.";
+
 /* ldap_search_s == ldap_search_st */
 
 /* ldap_ufn_search_c */
@@ -1089,6 +1364,7 @@ l_ldap_ufn_search_s( LDAPObject* self, PyObject* args )
     }
 }	
 
+
 /* ldap_ufn_setfilter */
 
 static PyObject*
@@ -1124,6 +1400,14 @@ l_ldap_ufn_setprefix( LDAPObject* self, PyObject* args )
     return Py_None;
 }
 
+static char doc_ufn[] =
+"ufn_setfilter(filtername) -> None\n"
+"ufn_setprefix(prefix) -> None\n"
+"ufn_search_s(url [,attrsonly=0])\n"
+"ufn_search_st(url [,attrsonly=0 [,timeout=-1]])\n\n"
+"\tSee the LDAP library manual pages for more information on these\n"
+"\t`user-friendly name' functions.";
+
 /* ldap_sort_entries */
 
 /* ldap_url_search */
@@ -1168,54 +1452,70 @@ l_ldap_url_search_st( LDAPObject* self, PyObject* args )
     }
 }
 
+static char doc_url_search[] =
+"url_search_s(url [,attrsonly=0])\n"
+"url_search_s(url [,attrsonly=0 [,timeout=-1]])\n\n"
+"\tThese routine works much like search_s*, except that many\n"
+"\tsearch parameters are pulled out of the URL url.\n"
+"\n"
+"\tLDAP URLs look like this:\n"
+"\t    ldap://host[:port]/dn[?attributes[?scope[?filter]]]\n"
+"\n"
+"\twhere scope is one of 'base' (default), 'one' or 'sub',\n"
+"\tand attributes is a comma-separated list of attributes to\n"
+"\tbe retrieved.\n"
+"\n"
+"\tURLs wrapped in angle-brackets and/or preceded by 'URL:'\n"
+"\tare tolerated.";
+
 /* methods */
 
 static PyMethodDef methods[] = {
-    {"unbind",		(PyCFunction)l_ldap_unbind,		METH_VARARGS},	
-    {"unbind_s",	(PyCFunction)l_ldap_unbind,		METH_VARARGS},	
-    {"abandon",		(PyCFunction)l_ldap_abandon,		METH_VARARGS},	
-    {"add",		(PyCFunction)l_ldap_add,		METH_VARARGS},	
-    {"add_s",		(PyCFunction)l_ldap_add_s,		METH_VARARGS},	
-    {"bind",		(PyCFunction)l_ldap_bind,		METH_VARARGS},	
-    {"bind_s",		(PyCFunction)l_ldap_bind_s,		METH_VARARGS},	
+    {"unbind",		(PyCFunction)l_ldap_unbind,		METH_VARARGS,	doc_unbind},	
+    {"unbind_s",	(PyCFunction)l_ldap_unbind,		METH_VARARGS,	doc_unbind},	
+    {"abandon",		(PyCFunction)l_ldap_abandon,		METH_VARARGS,	doc_abandon},	
+    {"add",		(PyCFunction)l_ldap_add,		METH_VARARGS,	doc_add},	
+    {"add_s",		(PyCFunction)l_ldap_add_s,		METH_VARARGS,	doc_add},	
+    {"bind",		(PyCFunction)l_ldap_bind,		METH_VARARGS,	doc_bind},	
+    {"bind_s",		(PyCFunction)l_ldap_bind_s,		METH_VARARGS,	doc_bind},	
 #ifdef LDAP_REFERRALS
-    {"set_rebind_proc",	(PyCFunction)l_ldap_set_rebind_proc,	METH_VARARGS},	
+    {"set_rebind_proc",	(PyCFunction)l_ldap_set_rebind_proc,	METH_VARARGS,	doc_set_rebind_proc},	
 #endif /* LDAP_REFERRALS */
-    {"simple_bind",	(PyCFunction)l_ldap_simple_bind,	METH_VARARGS},	
-    {"simple_bind_s",	(PyCFunction)l_ldap_simple_bind_s,	METH_VARARGS},	
+    {"simple_bind",	(PyCFunction)l_ldap_simple_bind,	METH_VARARGS,	doc_bind},	
+    {"simple_bind_s",	(PyCFunction)l_ldap_simple_bind_s,	METH_VARARGS,	doc_bind},	
 #ifdef WITH_KERBEROS
-    {"kerberos_bind_s",	(PyCFunction)l_ldap_kerberos_bind_s,	METH_VARARGS},	
-    {"kerberos_bind1",	(PyCFunction)l_ldap_kerberos_bind1,	METH_VARARGS},	
-    {"kerberos_bind1_s",(PyCFunction)l_ldap_kerberos_bind1_s,	METH_VARARGS},	
-    {"kerberos_bind2",	(PyCFunction)l_ldap_kerberos_bind2,	METH_VARARGS},	
-    {"kerberos_bind2_s",(PyCFunction)l_ldap_kerberos_bind2_s,	METH_VARARGS},	
+    {"kerberos_bind_s",	(PyCFunction)l_ldap_kerberos_bind_s,	METH_VARARGS,	doc_bind},	
+    {"kerberos_bind1",	(PyCFunction)l_ldap_kerberos_bind1,	METH_VARARGS,	doc_bind},	
+    {"kerberos_bind1_s",(PyCFunction)l_ldap_kerberos_bind1_s,	METH_VARARGS,	doc_bind},	
+    {"kerberos_bind2",	(PyCFunction)l_ldap_kerberos_bind2,	METH_VARARGS,	doc_bind},	
+    {"kerberos_bind2_s",(PyCFunction)l_ldap_kerberos_bind2_s,	METH_VARARGS,	doc_bind},	
 #endif /* WITH_KERBEROS */
 #ifndef NO_CACHE
-    {"enable_cache",	(PyCFunction)l_ldap_enable_cache,	METH_VARARGS},	
-    {"disable_cache",	(PyCFunction)l_ldap_disable_cache,	METH_VARARGS},	
-    {"set_cache_options",(PyCFunction)l_ldap_set_cache_options,	METH_VARARGS},	
-    {"destroy_cache",	(PyCFunction)l_ldap_destroy_cache,	METH_VARARGS},	
-    {"flush_cache",	(PyCFunction)l_ldap_flush_cache,	METH_VARARGS},	
-    {"uncache_entry",	(PyCFunction)l_ldap_uncache_entry,	METH_VARARGS},	
-    {"uncache_request",	(PyCFunction)l_ldap_uncache_request,	METH_VARARGS},	
+    {"enable_cache",	(PyCFunction)l_ldap_enable_cache,	METH_VARARGS,	doc_enable_cache},	
+    {"disable_cache",	(PyCFunction)l_ldap_disable_cache,	METH_VARARGS,	doc_disable_cache},	
+    {"set_cache_options",(PyCFunction)l_ldap_set_cache_options,	METH_VARARGS,	doc_set_cache_options},	
+    {"destroy_cache",	(PyCFunction)l_ldap_destroy_cache,	METH_VARARGS,	doc_destroy_cache},	
+    {"flush_cache",	(PyCFunction)l_ldap_flush_cache,	METH_VARARGS,	doc_flush_cache},	
+    {"uncache_entry",	(PyCFunction)l_ldap_uncache_entry,	METH_VARARGS,	doc_uncache_entry},	
+    {"uncache_request",	(PyCFunction)l_ldap_uncache_request,	METH_VARARGS,	doc_uncache_request},	
 #endif /* !NO_CACHE */
-    {"compare",		(PyCFunction)l_ldap_compare,		METH_VARARGS},	
-    {"compare_s",	(PyCFunction)l_ldap_compare_s,		METH_VARARGS},	
-    {"delete",		(PyCFunction)l_ldap_delete,		METH_VARARGS},	
-    {"delete_s",	(PyCFunction)l_ldap_delete_s,		METH_VARARGS},	
-    {"modify",		(PyCFunction)l_ldap_modify,		METH_VARARGS},	
-    {"modify_s",	(PyCFunction)l_ldap_modify_s,		METH_VARARGS},	
-    {"modrdn",		(PyCFunction)l_ldap_modrdn,		METH_VARARGS},	
-    {"modrdn_s",	(PyCFunction)l_ldap_modrdn_s,		METH_VARARGS},
-    {"result",		(PyCFunction)l_ldap_result,		METH_VARARGS},	
-    {"search",		(PyCFunction)l_ldap_search,		METH_VARARGS},	
-    {"search_s",	(PyCFunction)l_ldap_search_st,		METH_VARARGS},	
-    {"search_st",	(PyCFunction)l_ldap_search_st,		METH_VARARGS},	
-    {"ufn_search_s",	(PyCFunction)l_ldap_ufn_search_s,	METH_VARARGS},
-    {"ufn_setfilter",	(PyCFunction)l_ldap_ufn_setfilter,	METH_VARARGS},
-    {"ufn_setprefix",	(PyCFunction)l_ldap_ufn_setprefix,	METH_VARARGS},
-    {"url_search_s",	(PyCFunction)l_ldap_url_search_st,	METH_VARARGS},	
-    {"url_search_st",	(PyCFunction)l_ldap_url_search_st,	METH_VARARGS},	
+    {"compare",		(PyCFunction)l_ldap_compare,		METH_VARARGS,	doc_compare},	
+    {"compare_s",	(PyCFunction)l_ldap_compare_s,		METH_VARARGS,	doc_compare},	
+    {"delete",		(PyCFunction)l_ldap_delete,		METH_VARARGS,	doc_delete},	
+    {"delete_s",	(PyCFunction)l_ldap_delete_s,		METH_VARARGS,	doc_delete},	
+    {"modify",		(PyCFunction)l_ldap_modify,		METH_VARARGS,	doc_modify},	
+    {"modify_s",	(PyCFunction)l_ldap_modify_s,		METH_VARARGS,	doc_modify},	
+    {"modrdn",		(PyCFunction)l_ldap_modrdn,		METH_VARARGS,	doc_modrdn},	
+    {"modrdn_s",	(PyCFunction)l_ldap_modrdn_s,		METH_VARARGS,	doc_modrdn},
+    {"result",		(PyCFunction)l_ldap_result,		METH_VARARGS,	doc_result},	
+    {"search",		(PyCFunction)l_ldap_search,		METH_VARARGS,	doc_search},	
+    {"search_s",	(PyCFunction)l_ldap_search_st,		METH_VARARGS,	doc_search},	
+    {"search_st",	(PyCFunction)l_ldap_search_st,		METH_VARARGS,	doc_search},	
+    {"ufn_search_s",	(PyCFunction)l_ldap_ufn_search_s,	METH_VARARGS,	doc_ufn},
+    {"ufn_setfilter",	(PyCFunction)l_ldap_ufn_setfilter,	METH_VARARGS,	doc_ufn},
+    {"ufn_setprefix",	(PyCFunction)l_ldap_ufn_setprefix,	METH_VARARGS,	doc_ufn},
+    {"url_search_s",	(PyCFunction)l_ldap_url_search_st,	METH_VARARGS,	doc_url_search},	
+    {"url_search_st",	(PyCFunction)l_ldap_url_search_st,	METH_VARARGS,	doc_url_search},	
     { NULL, NULL }
 };
 
@@ -1227,14 +1527,14 @@ repr( LDAPObject* self )
     static char buf[4096];
 
 #   define STRFMT	"%s%s%s"
-#   define STRFMTP(s)                                                  \
-    		(s)==NULL?"":"'",                                       \
-		(s)==NULL?"(null)":(s),                                 \
+#   define STRFMTP(s)							\
+    		(s)==NULL?"":"'",					\
+		(s)==NULL?"None":(s),					\
 		(s)==NULL?"":"'"
 
 #   define LIMITFMT	"%d%s"
-#   define LIMITFMTP(v)                                                \
-    		(v),                                                    \
+#   define LIMITFMTP(v)							\
+    		(v),							\
 		(v)==LDAP_NO_LIMIT?" (NO_LIMIT)":""
     		
 
@@ -1292,10 +1592,18 @@ getattr( LDAPObject* self, char* name )
 		return PyInt_FromLong(self->ldap->ld_sizelimit);
 	if (streq(name,"errno")) 
 		return PyInt_FromLong(self->ldap->ld_errno);
-	if (streq(name,"error")) 
-		return PyString_FromString(self->ldap->ld_error);
-	if (streq(name,"matched")) 
-		return PyString_FromString(self->ldap->ld_matched);
+	if (streq(name,"error")) {
+		if (self->ldap->ld_error != NULL)
+			return PyString_FromString(self->ldap->ld_error);
+		Py_INCREF(Py_None);
+		return Py_None;
+	}
+	if (streq(name,"matched")) {
+		if (self->ldap->ld_matched != NULL)
+			return PyString_FromString(self->ldap->ld_matched);
+		Py_INCREF(Py_None);
+		return Py_None;
+	}
 	if (streq(name,"refhoplimit")) 
 		return PyInt_FromLong(self->ldap->ld_refhoplimit);
 	if (streq(name,"options")) 
@@ -1340,10 +1648,10 @@ setattr( LDAPObject* self, char* name, PyObject* value )
 	}
 
 	set(lberoptions, ~(unsigned char)0)
-	set(deref, ~0UL)
-	set(timelimit, ~0UL)
-	set(sizelimit, ~0UL)
-	set(refhoplimit, ~0UL)
+	set(deref, LONG_MAX)
+	set(timelimit, LONG_MAX)
+	set(sizelimit, LONG_MAX)
+	set(refhoplimit, LONG_MAX)
 	set(options, LDAP_OPT_RESTART)
 
 	/* it fell through to here */
@@ -1354,7 +1662,12 @@ setattr( LDAPObject* self, char* name, PyObject* value )
 /* type entry */
 
 PyTypeObject LDAP_Type = {
+#ifdef WIN32
+	/* see http://www.python.org/doc/FAQ.html#3.24 */
+	PyObject_HEAD_INIT(NULL)
+#else /* ! WIN32 */
 	PyObject_HEAD_INIT(&PyType_Type)
+#endif /* ! WIN32 */
 	0,                      /*ob_size*/
 	"LDAP",                 /*tp_name*/
 	sizeof(LDAPObject),     /*tp_basicsize*/
