@@ -1,15 +1,18 @@
+/* David Leonard <david.leonard@csee.uq.edu.au>, 1999. Public domain. */
 /* 
  * LDAP module
- * $Id: ldapmodule.c,v 1.1.1.1 1997/08/07 01:14:12 leonard Exp $
+ * $Id: ldapmodule.c,v 1.3 1999/09/19 01:53:21 leonard Exp $
  */
 
 #include "version.h"
+#include "common.h"
 #include "constants.h"
 #include "errors.h"
 #include "functions.h"
 /* #include "string_translators.h" */
 
 #include "Python.h"
+#include "LDAPObject.h"
 
 /* dummy module methods */
 
@@ -24,6 +27,11 @@ initldap()
 {
 	PyObject *m, *d;
 
+#ifdef WIN32
+	/* See http://www.python.org/doc/FAQ.html#3.24 */
+	LDAP_Type.ob_type = &PyType_Type;
+#endif
+
 	/* Create the module and add the functions */
 	m = Py_InitModule("ldap", methods);
 
